unit Childwin;

interface

uses
  SysUtils, Dialogs,
  Windows, Classes, Graphics, Forms, Controls, DBCtrls, StdCtrls, Mask,
  ExtCtrls,
  DataMod, Buttons;

const
  vc_NewFile = True;
  vc_OldFile = False;
  vc_SaveAs  = True;
  vc_Save    = False;
  vc_ReadOnly  = True;
  vc_NoReadOnly  = False;

type
  TMDIChild = class(TForm)
    Panel1: TPanel;
    DBNavigator: TDBNavigator;
    ScrollBox: TScrollBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    EditX_: TDBEdit;
    EditX_2: TDBEdit;
    EditY_: TDBEdit;
    EditY_2: TDBEdit;
    EditTHRESHOLD: TDBEdit;
    EditSTEP: TDBEdit;
    EditDEPTH: TDBEdit;
    EditMETHOD: TDBEdit;
    EditNAME: TDBEdit;
    ImageIMAGE: TDBImage;
    SaveDialog: TSaveDialog;
    ImageTMP: TImage;
    RepaintButton1: TSpeedButton;
    ColorDialog1: TColorDialog;
    BckButton: TSpeedButton;
    ImageBck: TImage;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ImageClickProc(Sender: TObject);
    procedure DBNavClickProc(Sender: TObject; Button: TNavigateBtn);
    procedure RepaintClickProc(Sender: TObject);
    procedure BckButtonClick(Sender: TObject);
    procedure FormShowSizeSet(Sender: TObject);
  private
    D_FullName : TFileName;
    D_FileName : TFileName;
    D_PathName : TFileName;
    D_Module   : TDataModule1;
    Is_New     : Boolean;
    Is_Saved   : Boolean;
    Is_ReadOnly: Boolean;
    CurrRec    : LongInt;
    NewRecord  : Boolean;
  public
    function DataLink_  (DBName: TFileName;
                         NewFile: Boolean;
                         ReadOnly: Boolean) : Boolean;
    function DataRename_(DBName: TFileName) : Boolean;
    procedure DataApply_;
    function FileSave_  (SaveAs: Boolean)   : Boolean;
    function Is_New_Fun      : Boolean;
    function Is_Saved_Fun    : Boolean;
    function Is_ReadOnly_Fun : Boolean;
    function NoEdit_    (ReadOnly : Boolean)   : Boolean;
    procedure CutItem_;
    procedure CopyItem_;
    procedure PasteItem_;
    procedure ImageBckShow(Color_ : TColor);
    function Is_D_Module      : TDataModule1;
    procedure PrintLookText;
    procedure PrintLook;
    procedure PrintForm;
  end;

implementation


uses
  mandbr, ExpForm, Main, Printers;

{$R *.DFM}



{======================================================================}
procedure TMDIChild.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if not Is_Saved and not Is_ReadOnly
  then
    case MessageDlg(D_FileName + ' is not saved.' +
                    #10#13 + 'Do You want to save it?',
                    mtConfirmation, [mbYes, mbNo, mbCancel], 0
                   )
    of
    mrYes : if FileSave_(Is_New) then Action := caFree
                                 else Action := caNone;
    mrNo  : begin
              if Is_New
              then begin
                D_Module.Table1.Close;
                D_Module.Table1.DeleteTable;
              end;
              Action := caFree;
            end;
    else Action := caNone;
    end {case}
  else Action := caFree;

  if Action = caFree then D_Module.Table1.Close;

end;

{======================================================================}
procedure TMDIChild.DataApply_;
begin
  D_Module.Table1.ApplyUpdates;
  D_Module.Table1.CommitUpdates;
end;

{======================================================================}
function TMDIChild.DataLink_(DBName : TFileName;
                            NewFile: Boolean;
                            ReadOnly: Boolean) : Boolean;
begin
  if ExtractFilePath(DBName) = ''
    then D_FullName := ExpandFileName(DBName)
    else D_FullName := DBName;
  D_PathName := ExtractFilePath(DBName);
  D_FileName := ExtractFileName(DBName);

  D_Module   := TDataModule1.Create(Application);
  D_Module.Table1.DataBaseName := D_PathName;
  D_Module.Table1.TableName    := D_FileName;

  DBNavigator.DataSource := D_Module.DataSource1;
  EditX_.DataSource := D_Module.DataSource1;
  EditX_2.DataSource := D_Module.DataSource1;
  EditY_.DataSource := D_Module.DataSource1;
  EditY_2.DataSource := D_Module.DataSource1;
  EditTHRESHOLD.DataSource := D_Module.DataSource1;
  EditSTEP.DataSource := D_Module.DataSource1;
  EditDEPTH.DataSource := D_Module.DataSource1;
  EditMETHOD.DataSource := D_Module.DataSource1;
  EditNAME.DataSource := D_Module.DataSource1;
  ImageIMAGE.DataSource := D_Module.DataSource1;

  if NewFile
  then begin
    D_Module.DataModuleNew(Application);
    D_Module.DataModuleInit(Application);
    Is_New := True;
  end
  else begin
    D_Module.DataModuleInit(Application);
    Is_New := False;
  end;
  Is_Saved := not Is_New;

  Is_ReadOnly := ReadOnly;
  NoEdit_(ReadOnly);


  if D_Module.Table1.RecordCount = 0
  then begin
    CurrRec := 0;
    RepaintButton1.Enabled := False;
    BckButton.Enabled := False;
  end
  else begin
    CurrRec := 1;
    ImageBckShow(TColor(D_Module.Table1BACKGROUND.Value));
    if (D_Module.Table1.State = dsInsert) or
       (D_Module.Table1.State = dsEdit)
    then begin
      RepaintButton1.Enabled := True;
      BckButton.Enabled := True;
    end
    else begin
      RepaintButton1.Enabled := False;
      BckButton.Enabled := False;
    end;
  end;
  Caption := '(' + IntToStr(CurrRec) + '/' +
             IntToStr(D_Module.Table1.RecordCount) + ') ' +
             D_FullName;

  NewRecord := False;
  DataLink_ := True;
end;



{======================================================================}
function TMDIChild.DataRename_(DBName: TFileName) : Boolean;
var
  IsActive : Boolean;
  Try_Name : String;
begin
  if Is_ReadOnly then
  begin
    ShowMessage(D_Fullname + #13#10 + 'It is readonly file.');
    DataRename_ := False;
    Exit;
  end;
  if FileExists(DBName)
  then
    case MessageDlg(DBName + ' is already exist.' +
                    #10#13 + ' Do You want to overwrite it?',
                    mtConfirmation, [mbYes, mbNo, mbCancel], 0
                   )
    of
    mrYes : ;
    else begin
           DataRename_ := False;
           Exit;
         end
    end; {case}


  if ExtractFilePath(DBName) = ''
    then Try_Name := ExpandFileName(DBName)
    else Try_Name := DBName;

  DataApply_;

  IsActive := D_Module.Table1.Active;
  D_Module.Table1.Active := False;
  D_Module.Table1.RenameTable(Try_Name);
  D_Module.Table1.Active := IsActive;

  D_FullName := Try_Name;
  D_PathName := ExtractFilePath(Try_Name);
  D_FileName := ExtractFileName(Try_Name);
  Caption := '(' + IntToStr(CurrRec) + '/' +
             IntToStr(D_Module.Table1.RecordCount) + ') ' +
             D_FullName;

  Is_New := False;
  DataRename_ := True;
end;


{======================================================================}
function TMDIChild.FileSave_(SaveAs: Boolean) : Boolean;
begin
  if Is_ReadOnly then
  begin
    ShowMessage(D_Fullname + #13#10 + 'It is readonly file.');
    FileSave_ := True;
    Exit;
  end;
  if Is_New or SaveAs then
  begin
    SaveDialog.Title := 'Save    '+D_FullName+'    As...';
    if SaveDialog.Execute
    then begin
      if DataRename_(SaveDialog.FileName)
      then begin
        Is_New := False;
      end
      else begin
        FileSave_ := False;
        Exit;
      end;
    end
    else begin
      FileSave_ := False;
      Exit;
    end;
  end;
  {Save...}
  DataApply_;
  Is_Saved := True;
  FileSave_ := True;
end;

{======================================================================}
function TMDIChild.Is_New_Fun : Boolean;
begin
  Is_New_Fun := Is_New;
end;

{======================================================================}
function TMDIChild.Is_Saved_Fun : Boolean;
begin
  Is_Saved_Fun := Is_Saved;
end;

{======================================================================}
function TMDIChild.Is_ReadOnly_Fun : Boolean;
begin
  Is_ReadOnly_Fun := Is_ReadOnly;
end;

{======================================================================}
function TMDIChild.NoEdit_(ReadOnly: Boolean) : Boolean;
begin
  EditX_.ReadOnly := ReadOnly;
  EditX_2.ReadOnly := ReadOnly;
  EditY_.ReadOnly := ReadOnly;
  EditY_2.ReadOnly := ReadOnly;
  EditTHRESHOLD.ReadOnly := ReadOnly;
  EditSTEP.ReadOnly := ReadOnly;
  EditDEPTH.ReadOnly := ReadOnly;
  EditMETHOD.ReadOnly := ReadOnly;
  EditNAME.ReadOnly := ReadOnly;
  ImageIMAGE.ReadOnly := ReadOnly;

  RepaintButton1.Enabled := not ReadOnly;
  BckButton.Enabled := not ReadOnly;

  NoEdit_ := True;
end;


{======================================================================}
procedure TMDIChild.ImageClickProc(Sender: TObject);
begin
  If StartExplore
  then begin
    DBNavigator.BtnClick(nbInsert);
  end
  else begin
    if (D_Module.Table1.State = dsInsert) or
       (D_Module.Table1.State = dsEdit)
    then begin
      DBNavigator.BtnClick(nbPost);
      DBNavigator.BtnClick(nbEdit);
    end
    else begin
      DBNavigator.BtnClick(nbEdit);
    end;
  end;
  Enabled := False;
  FormExp.Work;
end;

{======================================================================}
procedure TMDIChild.DBNavClickProc(Sender: TObject; Button: TNavigateBtn);
begin
  case Button of
    nbFirst  : begin
                 NewRecord := False;
                 if D_Module.Table1.RecordCount > 0
                 then CurrRec := 1
                 else CurrRec := 0;
               end;
    nbPrior  : begin
                 if NewRecord
                 then begin
                   NewRecord := False;
                   If not D_Module.Table1.Modified then Dec(CurrRec);
                 end;
                 if not D_Module.Table1.BOF then Dec(CurrRec);
               end;
    nbNext   : begin
                 if NewRecord
                 then begin
                   NewRecord := False;
                   If not D_Module.Table1.Modified then Dec(CurrRec);
                 end;
                 if not D_Module.Table1.EOF then Inc(CurrRec);
               end;
    nbLast   : begin
                 NewRecord := False;
                 CurrRec := D_Module.Table1.RecordCount;
               end;
    nbInsert : begin
                 NewRecord := True;
                 Is_Saved := False;
                 Inc(CurrRec);
               end;
    nbDelete : begin
                 if CurrRec > D_Module.Table1.RecordCount
                   then CurrRec := D_Module.Table1.RecordCount;
                 Is_Saved := False;
                 NewRecord := False;
               end;
    nbEdit   : Is_Saved := False;
    nbPost   : begin
                 NewRecord := False;
                 if CurrRec = 0 then CurrRec := 1;
                 Is_Saved := False;
               end;
    nbCancel : if NewRecord
               then begin
                 NewRecord := False;
                 Dec(CurrRec);
               end;
    nbRefresh: ;
  end;
  if NewRecord
  then begin
    Caption := '(new/' +
               IntToStr(D_Module.Table1.RecordCount) + ') ' +
               D_FullName;
    ImageBck.Hide;
  end
  else begin
    Caption := '(' + IntToStr(CurrRec) + '/' +
               IntToStr(D_Module.Table1.RecordCount) + ') ' +
               D_FullName;
    ImageBckShow(TColor(D_Module.Table1BACKGROUND.Value));
  end;
  if (D_Module.Table1.State = dsInsert) or
     (D_Module.Table1.State = dsEdit)
  then begin
    RepaintButton1.Enabled := True;
    BckButton.Enabled := True;
  end
  else begin
    RepaintButton1.Enabled := False;
    BckButton.Enabled := False;
  end

end;

{======================================================================}
procedure TMDIChild.CutItem_;
begin
  if ( ActiveControl = EditX_  )  or
     ( ActiveControl = EditX_2 )  or
     ( ActiveControl = EditY_  )  or
     ( ActiveControl = EditY_2 )  or
     ( ActiveControl = EditTHRESHOLD  )  or
     ( ActiveControl = EditSTEP       )  or
     ( ActiveControl = EditDEPTH      )  or
     ( ActiveControl = EditMETHOD     )  or
     ( ActiveControl = EditNAME       )
    then (ActiveControl as TDBEdit).CutToClipboard;
  if ActiveControl = ImageIMAGE
    then ImageIMAGE.CutToClipboard;
end;

{======================================================================}
procedure TMDIChild.CopyItem_;
begin
  if ( ActiveControl = EditX_  )  or
     ( ActiveControl = EditX_2 )  or
     ( ActiveControl = EditY_  )  or
     ( ActiveControl = EditY_2 )  or
     ( ActiveControl = EditTHRESHOLD  )  or
     ( ActiveControl = EditSTEP       )  or
     ( ActiveControl = EditDEPTH      )  or
     ( ActiveControl = EditMETHOD     )  or
     ( ActiveControl = EditNAME       )
    then (ActiveControl as TDBEdit).CopyToClipboard;
  if ActiveControl = ImageIMAGE
    then ImageIMAGE.CopyToClipboard;
end;

{======================================================================}
procedure TMDIChild.PasteItem_;
begin
  if ( ActiveControl = EditX_  )  or
     ( ActiveControl = EditX_2 )  or
     ( ActiveControl = EditY_  )  or
     ( ActiveControl = EditY_2 )  or
     ( ActiveControl = EditTHRESHOLD  )  or
     ( ActiveControl = EditSTEP       )  or
     ( ActiveControl = EditDEPTH      )  or
     ( ActiveControl = EditMETHOD     )  or
     ( ActiveControl = EditNAME       )
    then (ActiveControl as TDBEdit).PasteFromClipboard;
  if ActiveControl = ImageIMAGE
    then ImageIMAGE.PasteFromClipboard;
end;

{======================================================================}
procedure TMDIChild.RepaintClickProc(Sender: TObject);
var
  Im_X, Im_Y : Integer;
  D_Y        : Integer;
  Man : TMANDELBROT;
begin
  if (D_Module.Table1.State <> dsInsert) and
     (D_Module.Table1.State <> dsEdit)
    then Exit;
  {------------}begin
                  DBNavigator.BtnClick(nbPost);
                  DBNavigator.BtnClick(nbEdit);
                  ImageTMP.Left   := ImageIMAGE.Left;
                  ImageTMP.Top    := ImageIMAGE.Top;
                  ImageTMP.Width  := ImageIMAGE.Width;
                  ImageTMP.Height := ImageIMAGE.Height;
                  ImageTMP.Visible:= True;

                  ImageTMP.Canvas.Brush.Style := bsSolid;
                  ImageTMP.Canvas.Brush.Color := TColor(D_Module.Table1BACKGROUND.Value);
                  ImageTMP.Canvas.FloodFill( ImageTMP.Width div 2,
                             ImageTMP.Height div 2,
                             clBlack,
                             fsBorder);
                  ImageTMP.Canvas.FloodFill( ImageTMP.Width div 2,
                             ImageTMP.Height div 2,
                             clWhite,
                             fsBorder);
                  ImageTMP.Repaint;

                  Man := TMANDELBROT.Create;
                  If not Man.Init(D_Module.Table1X_0.Value,
                                   D_Module.Table1X_1.Value,
                                   D_Module.Table1Y_0.Value,
                                   D_Module.Table1Y_1.Value,
                                   ImageTMP.Width          ,
                                   ImageTMP.Height         ,
                                   D_Module.Table1METHOD.Value,
                                   D_Module.Table1BACKGROUND.Value,
                                   D_Module.Table1DEPTH.Value     ,
                                   D_Module.Table1THRESHOLD.Value ,
                                   D_Module.Table1STEP.Value
                                 )
                  then ShowMessage('Cannot init this look.')
                  else begin
                    ImageTMP.BringToFront;
                    ImageIMAGE.Visible := False;
                    D_Y := ImageIMAGE.Height - 1;
                    for Im_X:= 0 to ImageIMAGE.Width  - 1
                    do begin
                      for Im_Y:= D_Y downto 0 do
                        ImageTMP.Canvas.Pixels[Im_X,
                                               D_Y - Im_Y
                                              ] := Man.GetPixel(Im_X,Im_Y);
                      ImageTMP.Repaint;
                    end;
                    ImageTMP.SendToBack;
                  end;


                  D_Module.DataSource1.Edit;
                  D_Module.Table1Image.Assign(ImageTMP.Picture);
                  ImageTMP.Visible:= False;
                  ImageIMAGE.Visible := True;
  {------------}end
end;

{======================================================================}
procedure TMDIChild.BckButtonClick(Sender: TObject);
begin
  if (D_Module.Table1.State <> dsInsert) and
     (D_Module.Table1.State <> dsEdit)
    then Exit;
  if ColorDialog1.Execute
    then begin
      D_Module.Table1BACKGROUND.Value := ColorDialog1.Color;
      ImageBckShow(D_Module.Table1BACKGROUND.Value);
    end;
end;

{======================================================================}
procedure TMDIChild.ImageBckShow(Color_ : TColor);
begin
  ImageBck.Canvas.Brush.Style := bsSolid;
  ImageBck.Canvas.Brush.Color := Color_;
  ImageBck.Canvas.FloodFill( ImageBck.Width div 2,
                             ImageBck.Height div 2,
                             clWhite,
                             fsBorder);
  ImageBck.Canvas.FloodFill( ImageBck.Width div 2,
                             ImageBck.Height div 2,
                             clBlack,
                             fsBorder);
  ImageBck.Refresh;
  ImageBck.Show;
end;

{======================================================================}
function TMDIChild.Is_D_Module : TDataModule1;
begin
  Is_D_Module := D_Module;
end;

{======================================================================}
procedure TMDIChild.FormShowSizeSet(Sender: TObject);
begin
  Height := Panel1.Height + EditNAME.Top + EditNAME.Height + 50;
  Width  := ImageIMAGE.Left + ImageIMAGE.Width + 30;
end;

{======================================================================}
procedure TMDIChild.PrintLookText;
var
  MyFile: TextFile;
  OldPitch: TFontPitch;
  OldSize: Integer;
begin
  if NewRecord or (CurRRec = 0)
  then ShowMessage('Sorry, nothing to print...')
  else begin
    OldPitch := Printer.Canvas.Font.Pitch;
    OldSize  := Printer.Canvas.Font.Size;
    Printer.Canvas.Font.Pitch := fpFixed;
    Printer.Canvas.Font.Size  := 12;
    AssignPrn(MyFile);
    Rewrite(MyFile);
    Write(MyFile,
    '----------------------------------------------------',#13#10,
    'Database File: ',D_FullName,                  #13#10,
    'Look        #: ',CurrRec,                     #13#10,
    'Look Name    : ',EditNAME.Text,               #13#10,
    'X_0          : ',EditX_.Text,                 #13#10,
    'X_1          : ',EditX_2.Text,                #13#10,
    'Y_0          : ',EditY_.Text,                 #13#10,
    'Y_1          : ',EditY_2.Text,                #13#10,
    'THRESHOLD    :',EditTHRESHOLD.Text,           #13#10,
    'STEP         : ',EditSTEP.Text,               #13#10,
    'DEPTH        : ',EditDEPTH.Text,              #13#10,
    'METHOD       : ',EditMETHOD.Text,             #13#10,
    'BACKGROUND   : ',
        ColorToString(ImageBck.Canvas.Brush.Color),#13#10,
    '----------------------------------------------------',#13#10
           );
    System.CloseFile(MyFile);
    Printer.Canvas.Font.Pitch := OldPitch;
    Printer.Canvas.Font.Size  := OldSize;
  end;
end;

{======================================================================}
procedure TMDIChild.PrintLook;
var
  OldPitch: TFontPitch;
  OldSize: Integer;
  TopMg, LeftMg, DelMg : Integer;
begin
  if NewRecord or (CurRRec = 0)
  then ShowMessage('Sorry, nothing to print...')
  else begin
    OldPitch := Printer.Canvas.Font.Pitch;
    OldSize  := Printer.Canvas.Font.Size;
    Printer.Canvas.Font.Pitch := fpFixed;
    Printer.Canvas.Font.Size  := 12;
    LeftMg:= Printer.PageWidth div 10;
    TopMg := LeftMg;
    DelMg := Printer.Canvas.TextHeight('Victor') * 3;

    Printer.BeginDoc;
    Printer.Canvas.TextOut(LeftMg, TopMg,
    '-----------------------------------------------------------------');
    Printer.Canvas.TextOut(LeftMg, TopMg+DelMg*1,'Database File: '+D_FullName);
    Printer.Canvas.TextOut(LeftMg, TopMg+DelMg*2,'Look        #: '+IntToStr(CurrRec));
    Printer.Canvas.TextOut(LeftMg, TopMg+DelMg*3,'Look Name    : '+EditNAME.Text);
    Printer.Canvas.TextOut(LeftMg, TopMg+DelMg*4,'X_0          : '+EditX_.Text);
    Printer.Canvas.TextOut(LeftMg, TopMg+DelMg*5,'X_1          : '+EditX_2.Text);
    Printer.Canvas.TextOut(LeftMg, TopMg+DelMg*6,'Y_0          : '+EditY_.Text);
    Printer.Canvas.TextOut(LeftMg, TopMg+DelMg*7,'Y_1          : '+EditY_2.Text);
    Printer.Canvas.TextOut(LeftMg, TopMg+DelMg*8,'THRESHOLD    : '+EditTHRESHOLD.Text);
    Printer.Canvas.TextOut(LeftMg, TopMg+DelMg*9,'STEP         : '+EditSTEP.Text);
    Printer.Canvas.TextOut(LeftMg, TopMg+DelMg*10,'DEPTH        : '+EditDEPTH.Text);
    Printer.Canvas.TextOut(LeftMg, TopMg+DelMg*11,'METHOD       : '+EditMETHOD.Text);
    Printer.Canvas.TextOut(LeftMg, TopMg+DelMg*12,'BACKGROUND   : '+
      ColorToString(ImageBck.Canvas.Brush.Color));
    Printer.Canvas.TextOut(LeftMg, TopMg+DelMg*13,
    '-----------------------------------------------------------------');
    Printer.Canvas.Draw(LeftMg*7, TopMg+DelMg*4,
       ImageIMAGE.Picture.Graphic);
    Printer.Canvas.Draw(LeftMg*5, TopMg+DelMg*12,
       ImageBck.Picture.Graphic);
    Printer.EndDoc;

    Printer.Canvas.Font.Pitch := OldPitch;
    Printer.Canvas.Font.Size  := OldSize;
  end;
end;

{======================================================================}
procedure TMDIChild.PrintForm;
begin
    Print;
end;

end.
