unit EditImag;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, Dialogs;

type
  TEditDlg = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    Bevel1: TBevel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    BckButton: TSpeedButton;
    ImageBck: TImage;
    EditT: TEdit;
    EditS: TEdit;
    EditD: TEdit;
    EditM: TEdit;
    ColorDialog1: TColorDialog;
    procedure EditActivProc(Sender: TObject);
    procedure OKProc(Sender: TObject);
    procedure CancelProc(Sender: TObject);
    procedure BckClick(Sender: TObject);
  private
    METHOD             : String;
    BACKGROUND         : TColor;
    DEPTH,
    THRESHOLD,
    STEP               : Integer;
    Stat_              : Boolean;
  public
    procedure Init(M : String;
                   B : TColor;
                   D : Integer;
                   T : Integer;
                   S : Integer);
    procedure ReInit(var M : String;
                     var B : TColor;
                     var D : Integer;
                     var T : Integer;
                     var S : Integer);
    function Is_OK     : Boolean;
    procedure ImageBckShow(Color_ : TColor);
  end;
var
  EditDlg: TEditDlg;

implementation

uses
  mandbr;
{$R *.DFM}

{=========================================}
procedure TEditDlg.Init(M : String;
                        B : TColor;
                        D : Integer;
                        T : Integer;
                        S : Integer
                       );
begin
  METHOD     := M;
  BACKGROUND := B;
  DEPTH      := D;
  THRESHOLD  := T;
  STEP       := S;
  EditM.Text := METHOD;
  EditD.Text := IntToStr(DEPTH);
  EditT.Text := IntToStr(THRESHOLD);
  EditS.Text := IntToStr(STEP);
  ImageBckShow(BACKGROUND);
end;

{=========================================}
procedure TEditDlg.ReInit(var M : String;
                          var B : TColor;
                          var D : Integer;
                          var T : Integer;
                          var S : Integer
                         );
begin
  M := METHOD;
  B := BACKGROUND;
  D := DEPTH;
  T := THRESHOLD;
  S := STEP;
end;

{=========================================}
function TEditDlg.Is_OK     : Boolean;
begin
  Is_OK := Stat_;
end;

{=========================================}
procedure TEditDlg.EditActivProc(Sender: TObject);
begin
  Stat_ := False;
end;

{=========================================}
procedure TEditDlg.OKProc(Sender: TObject);
var
  ErrorMsg  : String;
  CodeD, CodeT, CodeS : Integer;
begin
  Stat_ := True;

  Val(EditD.Text, DEPTH,     CodeD);
  If CodeD<>0
  then begin
    ErrorMsg :=  ErrorMsg + EditD.Text + ' : Bad format' + #13#10;
    Stat_ := False;
  end;

  Val(EditT.Text, THRESHOLD, CodeT);
  If CodeT<>0
  then begin
    ErrorMsg :=  ErrorMsg + EditT.Text + ' : Bad format' + #13#10;
    Stat_ := False;
  end;

  Val(EditS.Text, STEP,      CodeS);
  If CodeS<>0
  then begin
    ErrorMsg :=  ErrorMsg + EditS.Text + ' : Bad format' + #13#10;
    Stat_ := False;
  end;

  if CodeD=0 then if DEPTH <= 0
  then begin
    ErrorMsg := ErrorMsg + #13#10 + 'DEPTH must be greater than 0';
    Stat_ := False;
  end;

  if CodeS=0 then if STEP <= 0
  then begin
    ErrorMsg := ErrorMsg + #13#10 + 'STEP must be greater than 0';
    Stat_ := False;
  end;

  if CodeT=0 then if THRESHOLD < 0
  then begin
    ErrorMsg := ErrorMsg + #13#10 +
                'THRESHOLD must be greater than 0 or equal to 0';
    Stat_ := False;
  end;

  if (CodeT=0) and (CodeD=0)
  then if THRESHOLD >= DEPTH
  then begin
    ErrorMsg := ErrorMsg + #13#10 +
                'THRESHOLD must be less than DEPTH';
    Stat_ := False;
  end;

  if (CodeT=0) and (CodeD=0) and (CodeS=0)
  then if STEP > (DEPTH - THRESHOLD)
  then begin
    ErrorMsg := ErrorMsg + #13#10 +
                'STEP must be less than DEPTH-THRESHOLD';
    Stat_ := False;
  end;

  METHOD      := EditM.Text;
  if not Is_AvMet(METHOD)
  then begin
    ErrorMsg := ErrorMsg + #13#10 + Get_AvMets;
    Stat_ := False;
  end;

  if not Stat_
  then ShowMessage(ErrorMsg)
  else Close;
end;

{=========================================}
procedure TEditDlg.CancelProc(Sender: TObject);
begin
  Stat_ := False;
  Close;
end;

{=========================================}
procedure TEditDlg.BckClick(Sender: TObject);
begin
  if ColorDialog1.Execute
  then begin
    BACKGROUND := ColorDialog1.Color;
    ImageBckShow(BACKGROUND);
  end;
end;

{======================================================================}
procedure TEditDlg.ImageBckShow(Color_ : TColor);
begin
  ImageBck.Canvas.Brush.Style := bsSolid;
  ImageBck.Canvas.Brush.Color := Color_;
  ImageBck.Canvas.FloodFill( ImageBck.Width div 2,
                             ImageBck.Height div 2,
                             clWhite,
                             fsBorder);
  ImageBck.Canvas.FloodFill( ImageBck.Width div 2,
                             ImageBck.Height div 2,
                             clBlack,
                             fsBorder);
  ImageBck.Refresh;
  ImageBck.Show;
end;


end.
