unit Main;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, Menus,
  StdCtrls, Dialogs, Buttons, Messages, ExtCtrls, ComCtrls, IniFiles;

const
  MyIniFileName :  TFileName = 'Universe.Ini';
  {!!!1/1/2 MaxReopenFiles:  Byte      = 4;}

type
  {!!!1/2/2 TReopenFiles = Array [1..MaxReopenFiles] of String;
   MaxReopenFiles isn't considered as const
  }
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    FileNewItem: TMenuItem;
    FileOpenItem: TMenuItem;
    FileCloseItem: TMenuItem;
    FileCloseAllItem: TMenuItem;
    Window1: TMenuItem;
    Help1: TMenuItem;
    N2: TMenuItem;
    FileN1MenuItem: TMenuItem;
    FileN2MenuItem: TMenuItem;
    FileN3MenuItem: TMenuItem;
    FileN4MenuItem: TMenuItem;
    N1: TMenuItem;
    FileExitItem: TMenuItem;
    WindowCascadeItem: TMenuItem;
    WindowTileItem: TMenuItem;
    WindowArrangeItem: TMenuItem;
    HelpAboutItem: TMenuItem;
    OpenDialog: TOpenDialog;
    FileSaveItem: TMenuItem;
    FileSaveAsItem: TMenuItem;
    Edit1: TMenuItem;
    CutItem: TMenuItem;
    CopyItem: TMenuItem;
    PasteItem: TMenuItem;
    WindowMinimizeItem: TMenuItem;
    SpeedPanel: TPanel;
    OpenBtn: TSpeedButton;
    SaveBtn: TSpeedButton;
    CutBtn: TSpeedButton;
    CopyBtn: TSpeedButton;
    PasteBtn: TSpeedButton;
    ExitBtn: TSpeedButton;
    StatusBar: TStatusBar;
    NewBtn: TSpeedButton;
    ExpBtn: TSpeedButton;
    FileExpItem: TMenuItem;
    N3: TMenuItem;
    Helpcontent1: TMenuItem;
    HelpIndex1: TMenuItem;
    Howtousehelp1: TMenuItem;
    N4: TMenuItem;
    PrinterSetupItem: TMenuItem;
    PrinterSetupDialog1: TPrinterSetupDialog;
    PrintItem: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FileNewItemClick(Sender: TObject);
    procedure WindowCascadeItemClick(Sender: TObject);
    procedure UpdateMenuItems(Sender: TObject);
    procedure WindowTileItemClick(Sender: TObject);
    procedure WindowArrangeItemClick(Sender: TObject);
    procedure FileCloseItemClick(Sender: TObject);
    procedure FileCloseAllItemClick(Sender: TObject);
    procedure FileOpenItemClick(Sender: TObject);
    procedure FileN1MenuItemClick(Sender: TObject);
    procedure FileN2MenuItemClick(Sender: TObject);
    procedure FileN3MenuItemClick(Sender: TObject);
    procedure FileN4MenuItemClick(Sender: TObject);
    procedure FileExitItemClick(Sender: TObject);
    procedure FileSaveItemClick(Sender: TObject);
    procedure FileSaveAsItemClick(Sender: TObject);
    procedure CutItemClick(Sender: TObject);
    procedure CopyItemClick(Sender: TObject);
    procedure PasteItemClick(Sender: TObject);
    procedure WindowMinimizeItemClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure HelpAboutItemClick(Sender: TObject);
    procedure ExpBtnClickProc(Sender: TObject);
    procedure HelpIndex1Click(Sender: TObject);
    procedure Helpcontent1Click(Sender: TObject);
    procedure Howtousehelp1Click(Sender: TObject);
    procedure PrinterSetupItemClick(Sender: TObject);
    procedure PrintItemClick(Sender: TObject);
  private
    { Private declarations }
    CurrReopenFiles : Byte;
    ReopenFiles : Array [1..4] of String;
    FirstShow : Boolean;
    CurrNewFile : Integer;
    procedure CreateMDIChild(const Name: string;
                             NewFile : Boolean;
                             ReadOnly: Boolean);
    procedure ShowHint(Sender: TObject);
    procedure ReOpenFileNN(const Name: string);
    procedure UpdateReOpenFile(const Name: string);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;
  StartExplore : Boolean;
implementation

{$R *.DFM}

uses ChildWin, About, PrintBox;

{=============================================================}
procedure TMainForm.FormCreate(Sender: TObject);
begin
  Application.OnHint := ShowHint;
  Screen.OnActiveFormChange := UpdateMenuItems;
  FirstShow := True;
  StartExplore := False;
  CurrNewFile := 1;
end;

{=============================================================}
procedure TMainForm.ShowHint(Sender: TObject);
begin
  StatusBar.SimpleText := Application.Hint;
end;

{=============================================================}
procedure TMainForm.CreateMDIChild(const Name: string;
                                   NewFile : Boolean;
                                   ReadOnly: Boolean);
var
  Child: TMDIChild;
begin
  Child := TMDIChild.Create(Application);
  if ReadOnly then Child.Caption := Name + ' (Read only)'
              else Child.Caption := Name;
  Child.DataLink_(Name, NewFile, ReadOnly);
  Child.FormShowSizeSet(Application);
end;

{=============================================================}
procedure TMainForm.FileNewItemClick(Sender: TObject);
var
  NewName : String;
begin
  repeat
    NewName := 'NONAME' + IntToStr(CurrNewFile);
    Inc(CurrNewFile);
  until not FileExists(NewName);
  CreateMDIChild(NewName,vc_NewFile,vc_NoReadOnly);
end;

{=============================================================}
procedure TMainForm.FileOpenItemClick(Sender: TObject);
var
  I: Integer;
begin
  if OpenDialog.Execute then
  for I := 0 to OpenDialog.Files.Count-1 do
    CreateMDIChild(OpenDialog.Files[I],
                   vc_OldFile,
                   (ofReadOnly in OpenDialog.Options)
                  );
end;

{=============================================================}
procedure TMainForm.FileCloseItemClick(Sender: TObject);
begin
  if ActiveMDIChild <> nil then
    ActiveMDIChild.Close;
end;

{=============================================================}
procedure TMainForm.FileCloseAllItemClick(Sender: TObject);
var
  I: Integer;
begin
  for I := MDIChildCount-1 downto 0 do
    MDIChildren[I].Close;
end;

{=============================================================}
procedure TMainForm.FileSaveItemClick(Sender: TObject);
begin
  if (ActiveMDIChild as TMDIChild).FileSave_(vc_Save)
   then UpdateReOpenFile(ActiveMDIChild.Caption);
end;

{=============================================================}
procedure TMainForm.FileSaveAsItemClick(Sender: TObject);
begin
  if (ActiveMDIChild as TMDIChild).FileSave_(vc_SaveAs)
   then UpdateReOpenFile(ActiveMDIChild.Caption);
end;

{=============================================================}
procedure TMainForm.FileExitItemClick(Sender: TObject);
begin
  Close;
end;

{=============================================================}
procedure TMainForm.CutItemClick(Sender: TObject);
begin
  {cut selection to clipboard}
  if ActiveMDIChild <> nil
    then (ActiveMDIChild as TMDIChild).CutItem_;
end;

{=============================================================}
procedure TMainForm.CopyItemClick(Sender: TObject);
begin
  {copy selection to clipboard}
  if ActiveMDIChild <> nil
    then (ActiveMDIChild as TMDIChild).CopyItem_;
end;

{=============================================================}
procedure TMainForm.PasteItemClick(Sender: TObject);
begin
  {paste from clipboard}
    if ActiveMDIChild <> nil
    then (ActiveMDIChild as TMDIChild).PasteItem_;
end;

{=============================================================}
procedure TMainForm.WindowCascadeItemClick(Sender: TObject);
begin
  Cascade;
end;

{=============================================================}
procedure TMainForm.WindowTileItemClick(Sender: TObject);
begin
  Tile;
end;

{=============================================================}
procedure TMainForm.WindowArrangeItemClick(Sender: TObject);
begin
  ArrangeIcons;
end;

{=============================================================}
procedure TMainForm.WindowMinimizeItemClick(Sender: TObject);
var
  I: Integer;
begin
  { Must be done backwards through the MDIChildren array }
  for I := MDIChildCount - 1 downto 0 do
    MDIChildren[I].WindowState := wsMinimized;
end;

{=============================================================}
procedure TMainForm.UpdateMenuItems(Sender: TObject);
begin
  FileCloseItem.Enabled := MDIChildCount > 0;
  FileCloseAllItem.Enabled := MDIChildCount > 0;
  FileSaveItem.Enabled := MDIChildCount > 0;
  FileSaveAsItem.Enabled := MDIChildCount > 0;
  PrintItem.Enabled := MDIChildCount > 0;
  CutItem.Enabled := MDIChildCount > 0;
  CopyItem.Enabled := MDIChildCount > 0;
  PasteItem.Enabled := MDIChildCount > 0;
  SaveBtn.Enabled := MDIChildCount > 0;
  CutBtn.Enabled := MDIChildCount > 0;
  CopyBtn.Enabled := MDIChildCount > 0;
  PasteBtn.Enabled := MDIChildCount > 0;
  WindowCascadeItem.Enabled := MDIChildCount > 0;
  WindowTileItem.Enabled := MDIChildCount > 0;
  WindowArrangeItem.Enabled := MDIChildCount > 0;
  WindowMinimizeItem.Enabled := MDIChildCount > 0;
end;

{=============================================================}
procedure TMainForm.FormDestroy(Sender: TObject);
begin
  Screen.OnActiveFormChange := nil;
end;

{=============================================================}
procedure TMainForm.FormActivate(Sender: TObject);
var   MyIni : TIniFile;
      i     : Byte;
begin
  if not FirstShow then Exit;
  FirstShow := False;
  MyIni := TIniFile.Create(MyIniFileName);
  with MyIni do
  begin
    Left  := ReadInteger('MainWindow', 'Left',  100);
    Top   := ReadInteger('MainWindow', 'Top',    50);
    Width := ReadInteger('MainWindow', 'Width', 250);
    Height:= ReadInteger('MainWindow', 'Height',250);
    for i:=1 to 4 do ReopenFiles[i] := '';
    ReopenFiles[1] :=  ReadString('Files', 'File1', '');
    ReopenFiles[2] :=  ReadString('Files', 'File2', '');
    ReopenFiles[3] :=  ReadString('Files', 'File3', '');
    ReopenFiles[4] :=  ReadString('Files', 'File4', '');
    CurrReopenFiles := ReadInteger('Files', 'Number',4);
    If CurrReopenFiles > 4 then CurrReopenFiles := 4;
    for i:=CurrReopenFiles+1 to 4 do ReopenFiles[i] := '';
    FileN1MenuItem.Caption := '&1. '+ ReopenFiles[1];
    FileN2MenuItem.Caption := '&2. '+ ReopenFiles[2];
    FileN3MenuItem.Caption := '&3. '+ ReopenFiles[3];
    FileN4MenuItem.Caption := '&4. '+ ReopenFiles[4];
    FileN1MenuItem.Visible := FileN1MenuItem.Caption <> '&1. ';
    FileN2MenuItem.Visible := FileN2MenuItem.Caption <> '&2. ';
    FileN3MenuItem.Visible := FileN3MenuItem.Caption <> '&3. ';
    FileN4MenuItem.Visible := FileN4MenuItem.Caption <> '&4. ';
    N2.Visible := FileN1MenuItem.Visible or
                  FileN2MenuItem.Visible or
                  FileN3MenuItem.Visible or
                  FileN4MenuItem.Visible;
  end;
  MyIni.Free;
  if ParamCount > 0 then CreateMDIChild(ParamStr(1),vc_OldFile,
                                        vc_NoReadOnly);
end;

{=============================================================}
procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
var   MyIni: TIniFile;
begin
  FileCloseAllItemClick(Application);
  If ActiveMDIChild <> nil
  then begin
    Action := caNone;
    Exit;
  end;

  MyIni := TIniFile.Create(MyIniFileName);
  with MyIni do
  begin
    WriteInteger('MainWindow', 'Left',  Left  );
    WriteInteger('MainWindow', 'Top',   Top   );
    WriteInteger('MainWindow', 'Width', Width );
    WriteInteger('MainWindow', 'Height',Height);
    WriteString('Files', 'File1', ReopenFiles[1]);
    WriteString('Files', 'File2', ReopenFiles[2]);
    WriteString('Files', 'File3', ReopenFiles[3]);
    WriteString('Files', 'File4', ReopenFiles[4]);
    WriteInteger('Files', 'Number',CurrReopenFiles);
  end;
  MyIni.Free;
  Action := caFree;
end;

{=============================================================}
procedure TMainForm.FileN1MenuItemClick(Sender: TObject);
begin
  ReOpenFileNN(FileN1MenuItem.Caption);
end;

{=============================================================}
procedure TMainForm.FileN2MenuItemClick(Sender: TObject);
begin
  ReOpenFileNN(FileN2MenuItem.Caption);
end;

{=============================================================}
procedure TMainForm.FileN3MenuItemClick(Sender: TObject);
begin
  ReOpenFileNN(FileN3MenuItem.Caption);
end;

{=============================================================}
procedure TMainForm.FileN4MenuItemClick(Sender: TObject);
begin
  ReOpenFileNN(FileN4MenuItem.Caption);
end;

{=============================================================}
procedure TMainForm.ReOpenFileNN(const Name: string);
begin
   CreateMDIChild(Copy(Name,4,1000),
                  vc_OldFile,vc_NoReadOnly);
end;

{=============================================================}
procedure TMainForm.UpdateReOpenFile(const Name: string);
var i : Byte;
begin
  if CurrReopenFiles = 0 then Exit;
  for i:=CurrReopenFiles-1 downto 1
   do ReopenFiles[i+1] := ReopenFiles[i];
  ReopenFiles[1] := Name;
end;


{=============================================================}
procedure TMainForm.HelpAboutItemClick(Sender: TObject);
begin
  AboutBox.ShowModal
end;

{=============================================================}
procedure TMainForm.ExpBtnClickProc(Sender: TObject);
begin
  if ActiveMDIChild = nil
  then begin
    FileNewItemClick(Application);
    StartExplore := True;
  end
  else StartExplore := False;
  if ActiveMDIChild <> nil
    then (ActiveMDIChild as TMDIChild).ImageClickProc(Application)
    else ShowMessage('Sorry, cannot start now');
end;

procedure TMainForm.HelpIndex1Click(Sender: TObject);
const
  EmptyString: PChar = '';
begin
  Application.HelpCommand(HELP_PARTIALKEY, Longint(EmptyString));
end;

procedure TMainForm.Helpcontent1Click(Sender: TObject);
begin
  Application.HelpCommand(HELP_CONTENTS, 0);
end;

procedure TMainForm.Howtousehelp1Click(Sender: TObject);
begin
  Application.HelpCommand(HELP_HELPONHELP, 0);
end;

procedure TMainForm.PrinterSetupItemClick(Sender: TObject);
begin
  PrinterSetupDialog1.Execute;
end;

procedure TMainForm.PrintItemClick(Sender: TObject);
begin
  if ActiveMDIChild <> nil
    then begin
      PrnDlg.ShowModal;
      If PrnDlg.ModalResult = mrOK
      then begin
        if PrnDlg.Radiobutton1.Checked
           then (ActiveMDIChild as TMDIChild).PrintForm;
        if PrnDlg.Radiobutton2.Checked
           then (ActiveMDIChild as TMDIChild).PrintLookText;
        if PrnDlg.Radiobutton3.Checked
           then (ActiveMDIChild as TMDIChild).PrintLook;
      end
    end
    else ShowMessage('Sorry, nothing to print...');
end;

end.
